/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.checker.logic;


public final class And extends BinaryExpr {
	public And(Expr a, Expr b) {
		super(a, b);
	}

	@Override
	protected Expr develop(boolean cnf) {
		if(!cnf) { // if DNF
			// And(a, Or(g, h) ) ==> Or( And(~a, ~g), And(~a, ~h) )
			if(b instanceof Or) {
				Expr g = ((Or)b).getA().develop(cnf);
				Expr h = ((Or)b).getB().develop(cnf);

				return new Or(new And(a, g), new And(a, h));
			}

			// And( Or(g, h), b ) ==> Or( And(~g, ~b), And(~h, ~b) )
			if(a instanceof Or) {
				Expr g = ((Or)a).getA().develop(cnf);
				Expr h = ((Or)a).getB().develop(cnf);

				return new Or(new And(g, b), new And(h, b));
			}
		}

		// if CNF or if the above did not work
		return new And(a.develop(cnf), b.develop(cnf));
	}

	@Override
	public String toString() {
		return "(" + a + " && " + b + ")";
	}

	public NaryAnd flatten() {
		NaryAnd n = new NaryAnd();

		if(a instanceof And) n.append(((And)a).flatten());
		else {
			if(a instanceof Or) n.add(((Or)a).flatten());
			else n.add(a);
		}
		if(b instanceof And) n.append(((And)b).flatten());
		else {
			if(b instanceof Or) n.add(((Or)b).flatten());
			else n.add(a);
		}

		return n;
	}

	@Override
	public void accept(ExprVisitor visitor) {
		visitor.visit(this);
	}

	@Override
	public BinaryExpr create(Expr a, Expr b) {
		return new And(a, b);
	}

	@Override
	public Expr simplify() {
		Expr sA = a.simplify();
		Expr sB = b.simplify();

		if(FALSE.equals(sA) || FALSE.equals(sB)) return FALSE;
		if(TRUE.equals(sA)) return sB;
		if(TRUE.equals(sB)) return sA;
		return this;
	}

	@Override
	public boolean isTemporal() {
		return a.isTemporal() || b.isTemporal();
	}

	@Override
	public ExprBundle getBundle() {
		ExprBundle aB = a.getBundle();
		ExprBundle bB = b.getBundle();
		return new ExprBundle(
				aB.inner != null && bB.inner != null ? new And(aB.inner, bB.inner) : null,
				aB.outer != null && bB.outer != null ? new And(aB.outer, bB.outer) : null,
				aB.isBundled || bB.isBundled);
	}
}
