/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.checker.logic;


public final class BackTo extends BinaryExpr {

	public BackTo(Expr a, Expr b) {
		super(a, b);
	}

	@Override
	public void accept(ExprVisitor visitor) {
		visitor.visit(this);
	}

	@Override
	protected Expr develop(boolean cnf) {
		// don't develop temporal operators
		return this;
	}

	@Override
	public String toString() {
		return "(" + a + ") backto (" + b + ")";
	}

	@Override
	public BinaryExpr create(Expr a, Expr b) {
		return new BackTo(a, b);
	}

	@Override
	public Expr simplify() {
		Expr sA = a.simplify();
		Expr sB = b.simplify();

		if(TRUE.equals(sB)) return sA;
		if(FALSE.equals(sB)) return (new Always(sA)).simplify();
		if(FALSE.equals(sA)) return FALSE;
		if(TRUE.equals(sA)) return TRUE;
		return this;
	}

	@Override
	public boolean isTemporal() {
		return true;
	}

	@Override
	public ExprBundle getBundle() {
		ExprBundle aB = a.getBundle();
		ExprBundle bB = b.getBundle();
		return new ExprBundle(
				aB.inner != null && bB.inner != null ? new BackTo(aB.inner, bB.inner) : null,
				aB.outer != null && bB.outer != null ? new BackTo(aB.outer, bB.outer) : null,
				aB.isBundled || bB.isBundled);
	}
}
