/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.checker.logic;

import java.util.HashSet;
import java.util.Set;

import fr.supelec.adlv.checker.PortInstance;

public class Constant extends Predicate {
	private final boolean value;

	public Constant(boolean value) {
		this.value = value;
	}

	public boolean getValue() {
		return value;
	}


	@Override
	public void accept(ExprVisitor visitor) {
		visitor.visit(this);
	}


	@Override
	public String toString() {
		return Boolean.toString(value);
	}

	@Override
	public Predicate neg() {
		return new Constant(!value);
	}

	@Override
	public boolean isNegationOf(Predicate p) {
		if(p instanceof Constant) {
			Constant cp = (Constant)p;
			return cp.value != value;
		}
		return false;
	}

	@Override
	public Set<PortInstance> getInputSignals() {
		return new HashSet<PortInstance>();
	}

}
