/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.checker.logic;


public final class Implies extends BinaryExpr {
	@Override
	public Expr develop(boolean cnf) {
		return new Or(new Not(a), b);
	}

	public Implies(Expr a, Expr b) {
		super(a, b);
	}

	@Override
	public String toString() {
		return "(" + a + " => " + b + ")";
	}

	@Override
	public void accept(ExprVisitor visitor) {
		visitor.visit(this);
	}

	@Override
	public BinaryExpr create(Expr a, Expr b) {
		return new Implies(a, b);
	}

	@Override
	public Expr simplify() {
		Expr sA = a.simplify();
		Expr sB = b.simplify();

		if(TRUE.equals(sA)) return sB;
		if(FALSE.equals(sA)) return TRUE;
		if(TRUE.equals(sB)) return TRUE;
		if(FALSE.equals(sB)) return (new Not(sA)).simplify();
		return new Implies(sA, sB);
	}

	@Override
	public boolean isTemporal() {
		return a.isTemporal() || b.isTemporal();
	}

	@Override
	public ExprBundle getBundle() {
		ExprBundle aB = a.getBundle();
		ExprBundle bB = b.getBundle();
		return new ExprBundle(
				aB.outer != null && bB.inner != null ? new Implies(aB.outer, bB.inner) : null,
				aB.inner != null && bB.outer != null ? new Implies(aB.inner, bB.outer) : null,
				aB.isBundled || bB.isBundled);
	}

}
