/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.checker.logic;


public final class Not extends UnaryExpr {
	@Override
	protected Expr develop(boolean cnf) {
		//	Not( And(a, b) ) ==> Or( Not(~a), Not(~b) )
		if(expr instanceof And) {
			Expr a = ((And)expr).getA().develop(cnf);
			Expr b = ((And)expr).getB().develop(cnf);

			return new Or(new Not(a), new Not(b));
		}

		// Not( Or(a, b) ) ==> And( Not(~a), Not(~b) )
		if(expr instanceof Or) {
			Expr a = ((Or)expr).getA().develop(cnf);
			Expr b = ((Or)expr).getB().develop(cnf);

			return new And(new Not(a), new Not(b));
		}

		// Not( predicate ) ==> negated predicate
		if(expr instanceof Predicate) {
			Predicate pred = (Predicate)expr;
			return pred.neg();
		}

		// else
		return new Not(expr.develop(cnf));
	}

	public Not(Expr e) {
		super(e);
	}

	@Override
	public boolean equals(Object o) {
		if(o instanceof Not) {
			return expr.equals(((Not)o).expr);
		} else return false;
	}

	@Override
	public String toString() {
		return "NOT(" + expr + ")";
	}

	@Override
	public void accept(ExprVisitor visitor) {
		visitor.visit(this);
	}

	@Override
	public UnaryExpr create(Expr a) {
		return new Not(a);
	}

	@Override
	public Expr simplify() {
		Expr sE = expr.simplify();

		if(sE instanceof Predicate) {
			return ((Predicate)sE).neg();
		}

		// DeMorgan's laws
		if(sE instanceof And) {
			And a = (And)sE;
			return new Or(new Not(a.getA()), new Not(a.getB()));
		}

		if(sE instanceof Or) {
			Or o = (Or)sE;
			return new And(new Not(o.getA()), new Not(o.getB()));
		}

		return new Not(sE);
	}

	@Override
	public boolean isTemporal() {
		return expr.isTemporal();
	}

	@Override
	public ExprBundle getBundle() {
		ExprBundle eB = expr.getBundle();
		return new ExprBundle(     // not: inversion of inner & outer intervals
				eB.outer != null ? new Not(eB.outer) : null,
				eB.inner != null ? new Not(eB.inner) : null,
				eB.isBundled);
	}
}
