/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.checker.logic;


public final class Or extends BinaryExpr {
	public Or(Expr a, Expr b) {
		super(a, b);
	}

	@Override
	protected Expr develop(boolean cnf) {
		if(cnf) { // if CNF
			// Or(a, And(g, h) ) ==> And( Or(a, g), Or(a, h) )
			if(b instanceof And) {
				Expr g = ((And)b).getA().develop(cnf);
				Expr h = ((And)b).getB().develop(cnf);

				return new And(new Or(a, g), new Or(a, h));
			}

			// Or( And(g, h), b ) ==> And( Or(~g, ~b), Or(~h, ~b) )
			if(a instanceof And) {
				Expr g = ((And)a).getA().develop(cnf);
				Expr h = ((And)a).getB().develop(cnf);

				return new And(new Or(g, b), new Or(h, b));
			}
		}

		// if DNF or if the above did not work
		return new Or(a.develop(cnf), b.develop(cnf));
	}

	@Override
	public String toString() {
		return "(" + a + " || " + b + ")";
	}

	public NaryOr flatten() {
		NaryOr n = new NaryOr();

		if(a instanceof Or) n.append(((Or)a).flatten());
		else {
			if(a instanceof And) n.add(((And)a).flatten());
			else n.add(a);
		}
		if(b instanceof Or) n.append(((Or)b).flatten());
		else {
			if(b instanceof And) n.add(((And)b).flatten());
			else n.add(b);
		}

		return n;
	}

	@Override
	public void accept(ExprVisitor visitor) {
		visitor.visit(this);
	}

	@Override
	public BinaryExpr create(Expr a, Expr b) {
		return new Or(a, b);
	}


	@Override
	public Expr simplify() {
		Expr sA = a.simplify();
		Expr sB = b.simplify();

		if(TRUE.equals(sA) || TRUE.equals(sB)) return TRUE;
		if(FALSE.equals(sA)) return sB;
		if(FALSE.equals(sB)) return sA;
		return new Or(sA, sB);
	}

	@Override
	public boolean isTemporal() {
		return a.isTemporal() || b.isTemporal();
	}

	@Override
	public ExprBundle getBundle() {
		ExprBundle aB = a.getBundle();
		ExprBundle bB = b.getBundle();
		return new ExprBundle(
				aB.inner != null && bB.inner != null ? new Or(aB.inner, bB.inner) : null,
				aB.outer != null && bB.outer != null ? new Or(aB.outer, bB.outer) : null,
				aB.isBundled || bB.isBundled);
	}
}
