/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.generator;

import java.io.File;
import java.io.IOException;

import fr.supelec.adlv.*;

import fr.supelec.adlv.parser.EmfAdlvHelper;

public class InternalComponentImplGenerator extends AdlvGenerator {
	InternalComponentImplGenerator( EmfAdlvHelper helper ) {
		helper_ = helper;
	}
	
	void generate( String resultsPath ) throws IOException {
		
		for( Component c : helper_.getApplication().getComponents() ) {
			ComponentType t = c.getType();
			if( t instanceof InternalComponentType ) {
				int sp = helper_.getComponentStreamProducerPorts( t ).size();
				int sc = helper_.getComponentStreamConsumerPorts( t ).size();
				int ep = helper_.getComponentEventProducerPorts( t ).size();
				int ec = helper_.getComponentEventConsumerPorts( t ).size();
				
				// We can work with the following kind of InternalComponents
				
				// Consume one Stream and produce Events
				if(( sp == 0 ) && ( sc == 1 ) && ( ep > 0 ) && ( ec == 0 )) {
					// ProducerPort must have associated expressions
					boolean ok = true;
					for( ProducerPort p : helper_.getComponentEventProducerPorts( t )) {
						if( p.getEventEmitExpression() == null ) {
							System.err.println( "ComponentType " + t.getName() + " missed at least one eventEmitxpression" );
							ok = false;
							break;
						}
					}
					if( ok ) {
						generateImplFilesForStreamToEvents( (InternalComponentType) t, resultsPath );
					}
				}
				
				// Consume one Stream and one Event and produce a Stream
				else if(( sp > 0 ) && ( sc > 0 ) && ( ep == 0 ) && ( ec > 0 )) {
					boolean ok = true;
					
					for( ConsumerPort p : helper_.getComponentEventConsumerPorts( t )) {
						if( p.getSetExpression() == null ) {
							System.err.println( "ComponentType " + t.getName() + " missed at least one setExpression" );
							ok = false;
							break;
						}
						if( p.getSetAttribute() == null ) {
							System.err.println( "ComponentType " + t.getName() + " missed at least one setAttribute" );
							ok = false;
							break;
						}
					}
					
					for( ProducerPort p : helper_.getComponentStreamProducerPorts( t )) {
						if( p.getOutputAttribute() == null ) {
							System.err.println( "ComponentType " + t.getName() + " missed at least one outputAttribute" );
							ok = false;
							break;
						}
					}

					if( ok ) {
						generateImplFilesForEventAndStreamToStream( (InternalComponentType) t, resultsPath );
					}
				}
				
				else {
					System.err.println( "ComponentType " + t.getName() + " cannot be generated" );
				}
			}
		}
	}
	
	private void generateImplFilesForStreamToEvents( InternalComponentType component, String resultsPath ) throws IOException {
		String cname = component.getName();
		String componentDirectory = resultsPath + "/" + cname;
		( new File( componentDirectory ) ).mkdir();
		
		String impl = cname + "_impl";

		openFile( componentDirectory, impl, HPP_EXTENSION );
		
		writeCppHeaderPrelude( component.getModule().getName() );
		
		out_.write( "#include \"microccm_" + cname + "_executor." + HPP_EXTENSION + "\"\n" );
		out_.write( "#include \"microccm_" + cname + "_CORBA." + HPP_EXTENSION + "\"\n" );
		out_.write( "\n" );
		
		out_.write( "class CCM_" + impl + "\n" );
		out_.write( "    : public CCM_" + cname + "\n" );
		out_.write( "    , public CCM_" + cname + "_SessionComponent_impl\n" );
		out_.write( "{\n" );
		out_.write( "public:\n" );
		out_.write( "    CCM_" + impl + "();\n" );
		out_.write( "\n" );
		out_.write( "    void configuration_complete()\n" );
		out_.write( "        throw( Components::InvalidConfiguration ) {}\n" );
		out_.write( "\n" );
		out_.write( "    void shutdown() {}\n" );
		out_.write( "\n" );

		for( ProducerPort p : helper_.getComponentStreamProducerPorts( component )) {
			// source
			StreamType s = (StreamType) p.getTokenType();
			out_.write( "    // GENERATED FROM : source " + s.getName() + " "
					  + p.getName() + ";\n" );
			out_.write( "    // Nothing generated\n" );
			out_.write( "\n" );
		}
		
		for( ConsumerPort p : helper_.getComponentStreamConsumerPorts( component )) {
			// sink
			StreamType s = (StreamType) p.getTokenType();
			out_.write( "    // GENERATED FROM : sink " + s.getName() + " "
					  + p.getName() + ";\n" );
			out_.write( "public:\n" );
			out_.write( "    void new_" + p.getName() + "_value( "
					  + getCorbaType( s.getType().getName() ) +" value );\n" );
			//out_.write( "    " + STI_MODULE_NAME + "::CCM_"
			out_.write( "    " + "CCM_"
					  + getStreamType( s.getType().getName() ) + "_ptr "
					  + "get_" + p.getName() + "() {\n" );
			out_.write( "        return &" + p.getName() + "_provider_;\n" );
			out_.write( "    }\n" );
			out_.write( "\n" );
			out_.write( "private:\n" );
			out_.write( "    struct Provides_" + p.getName() + "\n" );
			//out_.write( "        : " + STI_MODULE_NAME + "::CCM_"
			out_.write( "        : " + "CCM_"
				 	  + getStreamType( s.getType().getName() ) + "\n" );
			out_.write( "    {\n" );
			out_.write( "        Provides_" + p.getName()
					  + "( CCM_" + impl + " * c )\n" );
			out_.write( "            : component_( c ) {}\n" );
			out_.write( "\n" );
			out_.write( "        void put( "
					  + getCorbaType( s.getType().getName() )
					  + " value );\n" );
			out_.write( "\n" );
			out_.write( "        CCM_" + impl + " * component_;\n" );
			out_.write( "    };\n" );
			out_.write( "\n" );
			out_.write( "    Provides_" + p.getName() + " "
					  + p.getName() + "_provider_;\n" );
			out_.write( "\n" );
		}
		
		for( ProducerPort p : helper_.getComponentEventProducerPorts( component )) {
			// publishes
			EventType e = (EventType) p.getTokenType();
			out_.write( "    // GENERATED FROM : publishes " + e.getName() + " "
					  + p.getName() + ";\n" );
			if( p.getEventEmitExpression() != null ) {
				out_.write( "private:\n" );
				out_.write( "    bool previous_" + p.getName() + "_;\n" );
				out_.write( "\n" );
			}
		}
		
		for( ConsumerPort p : helper_.getComponentEventConsumerPorts( component )) {
			// consumes
			EventType e = (EventType) p.getTokenType();
			out_.write( "    // GENERATED FROM : consumes " + e.getName() + " "
					  + p.getName() + ";\n" );
			out_.write( "    // Nothing generated\n" );
			out_.write( "\n" );
		}

		out_.write( "\n" );
		out_.write( "};\n" );
		out_.write( "\n" );

		out_.write( "//---------------------------------------------------------------------\n\n" );

		String home = cname + "Home";
		String home_impl = home + "_impl";

		out_.write( "class " + home_impl + "\n" );
		out_.write( "    : public CCM_" + home + "\n" );
		out_.write( "{\n" );
		out_.write( "public:\n" );
		out_.write( "    Components::EnterpriseComponent_ptr create()\n" );
		out_.write( "        throw( Components::CCMException );\n" );
		out_.write( "};\n" );
		out_.write( "\n" );
		
		out_.write( "extern \"C\"\n" );
		out_.write( "{\n" );
		out_.write( "    Components::HomeExecutorBase * create_" + cname + "_home();\n" );
		out_.write( "}\n" );
		out_.write( "\n" );

		writeCppHeaderPostlude();
		
		closeFile();
		
		openFile( componentDirectory, impl, CPP_EXTENSION );
		
		writeCppCopyrightComment();
		
		out_.write( "#include \"" + impl + "." + HPP_EXTENSION + "\"\n" );
		out_.write( "\n" );

		out_.write( "CCM_" + impl + "::CCM_" + impl + "()\n" );
		boolean first_port = true;
		
		// Constructor
		//for( ProducerPort p : helper_.getComponentStreamProducerPorts( component )) {
			// source
			//StreamType s = (StreamType) p.getTokenType();
		//}
		
		for( ConsumerPort p : helper_.getComponentStreamConsumerPorts( component )) {
			// sink
			//StreamType s = (StreamType) p.getTokenType();
			if( first_port ) {
				out_.write( "    : " );
				first_port = false;
			}
			else {
				out_.write( "    , " );
			}
			out_.write( p.getName() + "_provider_( this )\n" );
		}
		
		for( ProducerPort p : helper_.getComponentEventProducerPorts( component )) {
			// publishes
			//EventType e = (EventType) p.getTokenType();
			if( p.getEventEmitExpression() != null ) {
				if( first_port ) {
					out_.write( "    : " );
					first_port = false;
				}
				else {
					out_.write( "    , " );
				}
				out_.write( "previous_" + p.getName() + "_( false )\n" );
			}
		}
		
		//for( ConsumerPort p : helper_.getComponentEventConsumerPorts( component )) {
			// consumes
			//EventType e = (EventType) p.getTokenType();
		//}
		
		out_.write( "{\n" );
		out_.write( "}\n" );
		out_.write( "\n" );
		
		for( ConsumerPort p : helper_.getComponentStreamConsumerPorts( component )) {
			// sink
			StreamType s = (StreamType) p.getTokenType();
			out_.write( "void CCM_" + impl + "::new_"
					  + p.getName() + "_value( "
					  + getCorbaType( s.getType().getName() )
					  + " " + p.getName() + " )\n" );
			out_.write( "{\n" );
			
			for( ProducerPort p2 : helper_.getComponentEventProducerPorts( component )) {
				// publishes
				EventType e2 = (EventType) p2.getTokenType();
				Expression exp2 = p2.getEventEmitExpression();
				if( exp2 != null ) {
					out_.write( "    bool new_" + p2.getName() + " =\n" );
					out_.write( "        "
							  + helper_.getStringForExpression( exp2 )
							  + ";\n" );
					out_.write( "    if( ! previous_" + p2.getName()
							  + "_ && new_" + p2.getName() + " ) {\n" );
					out_.write( "#ifdef TRACE_ADLV\n" );
					out_.write( "        std::clog << \"TRACE_ADLV "
							  + ": component : " + component.getName()
							  + ", publishes : " + p2.getName()
							  + ", event sent\\n\";\n" );
					out_.write( "#endif\n" );
					out_.write( "\n" );
					out_.write( "        "
							  + e2.getModule().getName() + "::"
							  + e2.getName() + "_var evt = new OBV_"
							  + e2.getModule().getName() + "::"
							  + e2.getName() + "();\n" );
					out_.write( "        component_context_->push_"
							  + p2.getName() + "( evt.in() );\n" );
					out_.write( "    }\n" );
					out_.write( "    previous_" + p2.getName()
							  + "_ = new_" + p2.getName() + ";\n" );
					out_.write( "\n" );
				}
			}
			
			out_.write( "}\n" );
			out_.write( "\n" );
		}

		for( ConsumerPort p : helper_.getComponentStreamConsumerPorts( component )) {
			// sink
			StreamType s = (StreamType) p.getTokenType();
			out_.write( "void CCM_" + impl + "::Provides_" + p.getName()
					  + "::put( "
					  + getCorbaType( s.getType().getName() )
					  + " value )\n" );
			out_.write( "{\n" );
			out_.write( "#ifdef TRACE_ADLV\n" );
			out_.write( "    std::clog << \"TRACE_ADLV "
					  + ": component : " + component.getName()
					  + ", sink : " + p.getName()
					  + ", value received : \" << value << \"\\n\";\n" );
			out_.write( "#endif\n" );
			out_.write( "\n" );
			out_.write( "    component_->new_" + p.getName() + "_value( value );\n" );
			out_.write( "}\n" );
			out_.write( "\n" );
		}
		out_.write( "\n" );

		out_.write( "//---------------------------------------------------------------------\n\n" );

		out_.write( "Components::EnterpriseComponent_ptr\n" );
		out_.write( home_impl + "::create() throw( Components::CCMException )\n" );
		out_.write( "{\n" );
		out_.write( "    return new CCM_" + impl + "();\n" );
		out_.write( "}\n" );
		out_.write( "\n" );

		out_.write( "extern \"C\"\n" );
		out_.write( "{\n" );
		out_.write( "    Components::HomeExecutorBase * create_" + cname + "_home()\n" );
		out_.write( "    {\n" );
		out_.write( "        return new " + home_impl + "();\n" );
		out_.write( "    }\n" );
		out_.write( "}\n" );
		out_.write( "\n" );

		closeFile();
	}
	
	private void generateImplFilesForEventAndStreamToStream( InternalComponentType component, String resultsPath ) throws IOException {
		String cname = component.getName();
		String componentDirectory = resultsPath + "/" + cname;
		( new File( componentDirectory ) ).mkdir();
		
		String impl = cname + "_impl";

		openFile( componentDirectory, impl, HPP_EXTENSION );
		
		writeCppHeaderPrelude( component.getModule().getName() );
		
		out_.write( "#include \"microccm_" + cname + "_executor." + HPP_EXTENSION + "\"\n" );
		out_.write( "#include \"microccm_" + cname + "_CORBA." + HPP_EXTENSION + "\"\n" );
		out_.write( "\n" );
		
		out_.write( "class CCM_" + impl + "\n" );
		out_.write( "    : public CCM_" + cname + "\n" );
		out_.write( "    , public CCM_" + cname + "_SessionComponent_impl\n" );
		out_.write( "{\n" );
		out_.write( "public:\n" );
		out_.write( "    CCM_" + impl + "();\n" );
		out_.write( "\n" );
		out_.write( "    void configuration_complete()\n" );
		out_.write( "        throw( Components::InvalidConfiguration ) {}\n" );
		out_.write( "\n" );
		out_.write( "    void shutdown() {}\n" );
		out_.write( "\n" );

		for( Attribute a : component.getAttributes() ) {
			out_.write( "    // GENERATED FROM : " + a.getType().getName()
					  + " " + a.getName() + "\n" );
			out_.write( "private:\n" );
			out_.write( "    " + getCorbaType( a.getType().getName() )
					  + " " + a.getName() + "_;\n" );
			out_.write( "public:\n" );
			out_.write( "    virtual " + getCorbaType( a.getType().getName() )
					  + " " + a.getName() + "() { return " + a.getName() + "_; }\n" );
			out_.write( "    virtual void " + a.getName() + "( "
					  + getCorbaType( a.getType().getName() )
					  + " attribute ) { " + a.getName() + "_ = attribute; }\n" );
			out_.write( "\n" );
		}
		for( ProducerPort p : helper_.getComponentStreamProducerPorts( component )) {
			// source
			StreamType s = (StreamType) p.getTokenType();
			out_.write( "    // GENERATED FROM : source " + s.getName() + " "
					  + p.getName() + ";\n" );
			out_.write( "    // Nothing generated\n" );
			out_.write( "\n" );
		}
		
		for( ConsumerPort p : helper_.getComponentStreamConsumerPorts( component )) {
			// sink
			StreamType s = (StreamType) p.getTokenType();
			out_.write( "    // GENERATED FROM : sink " + s.getName() + " "
					  + p.getName() + ";\n" );
			out_.write( "public:\n" );
			out_.write( "    void new_" + p.getName() + "_value( "
					  + getCorbaType( s.getType().getName() ) +" value );\n" );
			//out_.write( "    " + STI_MODULE_NAME + "::CCM_"
			out_.write( "    " + "CCM_"
					  + getStreamType( s.getType().getName() ) + "_ptr "
					  + "get_" + p.getName() + "() {\n" );
			out_.write( "        return &" + p.getName() + "_provider_;\n" );
			out_.write( "    }\n" );
			out_.write( "\n" );
			out_.write( "private:\n" );
			out_.write( "    " + getCorbaType( s.getType().getName() )
					  + " " + p.getName() + ";\n" );
			out_.write( "\n" );
			out_.write( "    struct Provides_" + p.getName() + "\n" );
			//out_.write( "        : " + STI_MODULE_NAME + "::CCM_"
			out_.write( "        : " + "CCM_"
					  + getStreamType( s.getType().getName() ) + "\n" );
			out_.write( "    {\n" );
			out_.write( "        Provides_" + p.getName()
					 + "( CCM_" + impl + " * c )\n" );
			out_.write( "            : component_( c ) {}\n" );
			out_.write( "\n" );
			out_.write( "        void put( "
					  + getCorbaType( s.getType().getName() )
					  + " value );\n" );
			out_.write( "\n" );
			out_.write( "        CCM_" + impl + " * component_;\n" );
			out_.write( "    };\n" );
			out_.write( "\n" );
			out_.write( "    Provides_" + p.getName() + " "
					  + p.getName() + "_provider_;\n" );
			out_.write( "\n" );
		}
		
		for( ProducerPort p : helper_.getComponentEventProducerPorts( component )) {
			// publishes
			EventType e = (EventType) p.getTokenType();
			out_.write( "    // GENERATED FROM : publishes " + e.getName() + " "
					  + p.getName() + ";\n" );
			out_.write( "    // Nothing generated\n" );
			out_.write( "\n" );
		}
		
		for( ConsumerPort p : helper_.getComponentEventConsumerPorts( component )) {
			// consumes
			EventType e = (EventType) p.getTokenType();
			out_.write( "    // GENERATED FROM : consumes " + e.getName() + " "
					  + p.getName() + ";\n" );
			out_.write( "    void push_" + p.getName() + "( "
					  + e.getModule().getName() + "::"
					  + e.getName() + " * evt );\n" );
			out_.write( "\n" );
		}

		out_.write( "\n" );
		out_.write( "};\n" );
		out_.write( "\n" );

		out_.write( "//---------------------------------------------------------------------\n\n" );

		String home = cname + "Home";
		String home_impl = home + "_impl";

		out_.write( "class " + home_impl + "\n" );
		out_.write( "    : public CCM_" + home + "\n" );
		out_.write( "{\n" );
		out_.write( "public:\n" );
		out_.write( "    Components::EnterpriseComponent_ptr create()\n" );
		out_.write( "        throw( Components::CCMException );\n" );
		out_.write( "};\n" );
		out_.write( "\n" );
		
		out_.write( "extern \"C\"\n" );
		out_.write( "{\n" );
		out_.write( "    Components::HomeExecutorBase * create_" + cname + "_home();\n" );
		out_.write( "}\n" );
		out_.write( "\n" );

		writeCppHeaderPostlude();
		
		closeFile();
		
		openFile( componentDirectory, impl, CPP_EXTENSION );
		
		writeCppCopyrightComment();
		
		out_.write( "#include \"" + impl + "." + HPP_EXTENSION + "\"\n" );
		out_.write( "\n" );

		// Constructor

		out_.write( "CCM_" + impl + "::CCM_" + impl + "()\n" );
		boolean first_port = true;
		
		for( Attribute a : component.getAttributes() ) {
			if( first_port ) {
				out_.write( "    : " );
				first_port = false;
			}
			else {
				out_.write( "    , " );
			}
			out_.write( a.getName() + "_( 0 )\n" );
		}

		//for( ProducerPort p : helper_.getComponentStreamProducerPorts( component )) {
			// source
			//StreamType s = (StreamType) p.getTokenType();
		//}
		
		for( ConsumerPort p : helper_.getComponentStreamConsumerPorts( component )) {
			// sink
			//StreamType s = (StreamType) p.getTokenType();
			if( first_port ) {
				out_.write( "    : " );
				first_port = false;
			}
			else {
				out_.write( "    , " );
			}
			out_.write( p.getName() + "( 0 )\n" );
			out_.write( "    , " + p.getName() + "_provider_( this )\n" );
		}
		
		//for( ProducerPort p : helper_.getComponentEventProducerPorts( component )) {
			// publishes
			//EventType e = (EventType) p.getTokenType();
		//}
		
		//for( ConsumerPort p : helper_.getComponentEventConsumerPorts( component )) {
			// consumes
			//EventType e = (EventType) p.getTokenType();
		//}
		
		out_.write( "{\n" );
		out_.write( "}\n" );
		out_.write( "\n" );
		
		for( ConsumerPort p : helper_.getComponentStreamConsumerPorts( component )) {
			// sink
			StreamType s = (StreamType) p.getTokenType();
			out_.write( "void CCM_" + impl + "::new_"
					  + p.getName() + "_value( "
					  + getCorbaType( s.getType().getName() )
					  + " value )\n" );
			out_.write( "{\n" );
			
			out_.write( "#ifdef TRACE_ADLV\n" );
			out_.write( "    std::clog << \"TRACE_ADLV "
					  + ": component : " + component.getName()
					  + ", sink : " + p.getName()
					  + ", value received : \" << value << \"\\n\";\n" );
			out_.write( "#endif\n" );
			out_.write( "\n" );

			out_.write( "    " + p.getName() + " = value;\n" );
			out_.write( "\n" );

			out_.write( "    // We use the current flow of control to update our consumers\n" );
			for( ProducerPort p2 : helper_.getComponentStreamProducerPorts( component )) {
				Attribute a = p2.getOutputAttribute();
				if( a != null ) {
					out_.write( "    {\n" );
					StreamType s2 = (StreamType) p.getTokenType();
					//out_.write( "        " + STI_MODULE_NAME + "::"
					out_.write( "        "
							  + getStreamType( s2.getType().getName() )
							  + "_ptr cl = component_context_->get_connection_"
							  + p2.getName()
							  + "();\n" );
					
					out_.write( "        if( cl ) {\n" );
					out_.write( "#ifdef TRACE_ADLV\n" );
					out_.write( "            std::clog << \"TRACE_ADLV "
							  + ": component : " + component.getName()
							  + ", uses : " + p2.getName()
							  + ", value sent : \" << " + a.getName() + "() << \"\\n\";\n" );
					out_.write( "#endif\n" );
					out_.write( "\n" );

					out_.write( "            cl->put( " + a.getName() + "() );\n" );
					out_.write( "        }\n" );
					out_.write( "    }\n" );
				}
			}
			out_.write( "\n" );

			out_.write( "}\n" );
			out_.write( "\n" );
		}

		for( ConsumerPort p : helper_.getComponentStreamConsumerPorts( component )) {
			// sink
			StreamType s = (StreamType) p.getTokenType();
			out_.write( "void CCM_" + impl + "::Provides_" + p.getName()
					  + "::put( "
					  + getCorbaType( s.getType().getName() )
					  + " value )\n" );
			out_.write( "{\n" );
			out_.write( "    component_->new_" + p.getName() + "_value( value );\n" );
			out_.write( "}\n" );
			out_.write( "\n" );
		}
		out_.write( "\n" );

		for( ConsumerPort p : helper_.getComponentEventConsumerPorts( component )) {
			// consumes
			EventType e = (EventType) p.getTokenType();
			out_.write( "void CCM_" + impl + "::push_" + p.getName() + "( "
					  + e.getModule().getName() + "::"
					  + e.getName() + " * evt )\n" );
			out_.write( "{\n" );
			out_.write( "#ifdef TRACE_ADLV\n" );
			out_.write( "    std::clog << \"TRACE_ADLV "
					  + ": component : " + component.getName()
					  + ", consumes : " + p.getName()
					  + ", event received\\n\";\n" );
			out_.write( "#endif\n" );
			out_.write( "\n" );
			out_.write( "    " + p.getSetAttribute().getName()
					  + "( " + helper_.getStringForExpression( p.getSetExpression() ) + " );\n" );
			out_.write( "}\n" );
			out_.write( "\n" );
		}
	
		out_.write( "//---------------------------------------------------------------------\n\n" );

		out_.write( "Components::EnterpriseComponent_ptr\n" );
		out_.write( home_impl + "::create() throw( Components::CCMException )\n" );
		out_.write( "{\n" );
		out_.write( "    return new CCM_" + impl + "();\n" );
		out_.write( "}\n" );
		out_.write( "\n" );

		out_.write( "extern \"C\"\n" );
		out_.write( "{\n" );
		out_.write( "    Components::HomeExecutorBase * create_" + cname + "_home()\n" );
		out_.write( "    {\n" );
		out_.write( "        return new " + home_impl + "();\n" );
		out_.write( "    }\n" );
		out_.write( "}\n" );
		out_.write( "\n" );

		closeFile();
	}

	private EmfAdlvHelper helper_;
}
