/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.parser;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import fr.supelec.adlv.ActiveExpression;
import fr.supelec.adlv.Application;
import fr.supelec.adlv.Attribute;
import fr.supelec.adlv.BinaryExpression;
import fr.supelec.adlv.BinaryOperator;
import fr.supelec.adlv.Component;
import fr.supelec.adlv.ComponentType;
import fr.supelec.adlv.ConsumerPort;
import fr.supelec.adlv.ControlComponentType;
import fr.supelec.adlv.EventType;
import fr.supelec.adlv.Expression;
import fr.supelec.adlv.InternalComponentType;
import fr.supelec.adlv.LiteralExpression;
import fr.supelec.adlv.Module;
import fr.supelec.adlv.Port;
import fr.supelec.adlv.PortExpression;
import fr.supelec.adlv.ProcessingComponentType;
import fr.supelec.adlv.ProducerPort;
import fr.supelec.adlv.SimpleExpression;
import fr.supelec.adlv.StreamType;
import fr.supelec.adlv.TokenType;
import fr.supelec.adlv.UnaryExpression;
import fr.supelec.adlv.UnaryOperator;



public class EmfAdlvHelper {

    public Iterator< String > moduleNameIterator() {
        return modules_.keySet().iterator();
    }

    public void addModule( Module module ) {
        modules_.put( module.getName(), module );
    }

    public Module getModule( String module_name ) {
        Module result = modules_.get( module_name );
        if( result == null ) {
            System.err.println("Error : Module " + module_name + " not found !" );
        }
        return result;
    }

    public ComponentType getComponentType( String module_name, String component_name ) {
        Module module = getModule( module_name );
        ComponentType result = null;
        for( ComponentType c : module.getComponentTypes() ) {
             if( c.getName().equals( component_name )) {
                result = c;
                break;
            }
        }
        if( result == null ) {
            System.err.println("Error : ComponentType " + component_name + " not found !" );
        }
        return result;
    }

    public ControlComponentType getControlComponentType( String module_name, String component_name ) {
        ComponentType result = getComponentType( module_name, component_name );
        if(( result != null ) && ( result instanceof ControlComponentType )) {
            return (ControlComponentType) result;
        }
        if( result != null ) {
            System.err.println("Error : ComponentType " + component_name + " is not a ControlComponent !" );
        }
        return null;
    }

    public ProcessingComponentType getProcessingComponentType( String module_name, String component_name ) {
        ComponentType result = getComponentType( module_name, component_name );
        if(( result != null ) && ( result instanceof ProcessingComponentType )) {
            return (ProcessingComponentType) result;
        }
        if( result != null ) {
            System.err.println("Error : ComponentType " + component_name + " is not a ProcessingComponent !" );
        }
        return null;
    }

    public InternalComponentType getInternalComponentType( String module_name, String component_name ) {
        ComponentType result = getComponentType( module_name, component_name );
        if(( result != null ) && ( result instanceof InternalComponentType )) {
            return (InternalComponentType) result;
        }
        if( result != null ) {
            System.err.println("Error : ComponentType " + component_name + " is not an InternalComponent !" );
        }
        return null;
    }

    public Component getApplicationComponent( String component_name ) {
        Component result = null;
        for( Component c : application_.getComponents() ) {
            if( c.getName().equals( component_name )) {
                result = c;
                break;
            }
        }
        if( result == null ) {
        	// Used in ConsumerPortExpression
            //System.err.println("Error : Component " + component_name + " not found !" );
        }
        return result;
    }

    public TokenType getTokenType( String module_name, String token_name ) {
        Module module = getModule( module_name );
        TokenType result = null;
        for( TokenType t : module.getTokenTypes() ) {
            if( t.getName().equals( token_name )) {
                result = t;
                break;
            }
        }
        if( result == null ) {
            System.err.println("Error : TokenType " + token_name + " not found !" );
        }
        return result;
    }

    public EventType getEventType( String module_name, String event_name ) {
        TokenType result = getTokenType( module_name, event_name );
        if(( result != null ) && ( result instanceof EventType )) {
            return (EventType) result;
        }
        if( result != null ) {
            System.err.println("Error : TokenType " + event_name + " is not an EventType !" );
        }
        return null;
    }

    public StreamType getStreamType( String module_name, String stream_name ) {
        TokenType result = getTokenType( module_name, stream_name );
        if(( result != null ) && ( result instanceof StreamType )) {
            return (StreamType) result;
        }
        if( result != null ) {
            System.err.println("Error : TokenType " + stream_name + " is not an StreamType !" );
        }
        return null;
    }

    public Port getApplicationPort( String port_name ) {
        Port result = null;
        for( Port p : application_.getPorts() ) {
            if( p.getName().equals( port_name )) {
                result = p;
                break;
            }
        }
        if( result == null ) {
            System.err.println("Error : Port " + port_name + " not found in application !" );
        }
        return result;
    }

    public ConsumerPort getApplicationConsumerPort( String port_name ) {
        Port result = getApplicationPort( port_name );
        if(( result != null ) && ( result instanceof ConsumerPort )) {
            return (ConsumerPort) result;
        }
        if( result != null ) {
            // used by isApplicationInternalPort
            //System.err.println("Error : Port " + port_name + " is not a ConsumerPort !" );
        }
        return null;
    }

    public ProducerPort getApplicationProducerPort( String port_name ) {
        Port result = getApplicationPort( port_name );
        if(( result != null ) && ( result instanceof ProducerPort )) {
            return (ProducerPort) result;
        }
        if( result != null ) {
            System.err.println("Error : Port " + port_name + " is not a ProducerPort !" );
        }
        return null;
    }

    public boolean isApplicationInternalPort( String port_name ) {
        ConsumerPort result = getApplicationConsumerPort( port_name );
        if( result == null ) {
            return false;
        }
        return ! result.getSteps().isEmpty();
    }

    public List< ConsumerPort > getApplicationConsumerPorts() {
        LinkedList< ConsumerPort > result = new LinkedList< ConsumerPort >();
        for( Port p : application_.getPorts() ) {
            if( p instanceof ConsumerPort ) {
                   result.add(( ConsumerPort ) p );
            }
        }
        return result;
    }

    public List< ConsumerPort > getApplicationInternalConsumerPorts() {
        LinkedList< ConsumerPort > result = new LinkedList< ConsumerPort >();
        for( Port p : application_.getPorts() ) {
            if( p instanceof ConsumerPort ) {
            	if( isApplicationInternalPort( p.getName() )) {
                   result.add(( ConsumerPort ) p );
            	}
            }
        }
        return result;
    }

    public List< ConsumerPort > getApplicationExternalConsumerPorts() {
        LinkedList< ConsumerPort > result = new LinkedList< ConsumerPort >();
        for( Port p : application_.getPorts() ) {
            if( p instanceof ConsumerPort ) {
            	if( ! isApplicationInternalPort( p.getName() )) {
                   result.add(( ConsumerPort ) p );
            	}
            }
        }
        return result;
    }

    public List< ProducerPort > getApplicationProducerPorts() {
        LinkedList< ProducerPort > result = new LinkedList< ProducerPort >();
        for( Port p : application_.getPorts() ) {
            if( p instanceof ProducerPort ) {
                result.add(( ProducerPort ) p );
            }
        }
        return result;
    }

    public List< ProducerPort > getApplicationExternalProducerPorts() {
        LinkedList< ProducerPort > result = new LinkedList< ProducerPort >();
        for( Port p : application_.getPorts() ) {
            if( p instanceof ProducerPort ) {
            	if( ! isApplicationInternalPort( p.getName() )) {
            		result.add(( ProducerPort ) p );
            	}
            }
        }
        return result;
    }

    public Attribute getComponentAttribute( ComponentType component, String attribute_name ) {
        InternalComponentType i = ( InternalComponentType ) component;
        for( Attribute a : i.getAttributes() ) {
            if( a.getName().equals( attribute_name )) return a;
        }
        return null;
    }

    public Port getComponentPort( ComponentType component, String port_name ) {
        Port result = null;
        for( Port p : component.getPorts() ) {
            if( p.getName().equals( port_name )) {
                result = p;
                break;
            }
        }
        if( result == null ) {
            System.err.println("Error : Port " + port_name + " not found in Component " + component.getName() );
        }
        return result;
    }

    public ConsumerPort getComponentConsumerPort( ComponentType component, String port_name ) {
        Port result = getComponentPort( component, port_name );
        if(( result != null ) && ( result instanceof ConsumerPort )) {
            return (ConsumerPort) result;
        }
        if( result != null ) {
            System.err.println("Error : Port " + port_name + " is not a ConsumerPort in Component " + component.getName() );
        }
        return null;
    }

    public ProducerPort getComponentProducerPort( ComponentType component, String port_name ) {
        Port result = getComponentPort( component, port_name );
        if(( result != null ) && ( result instanceof ProducerPort )) {
            return (ProducerPort) result;
        }
        if( result != null ) {
            System.err.println("Error : Port " + port_name + " is not a ProducerPort in Component " + component.getName() );
        }
        return null;
    }

    public List< ConsumerPort > getComponentConsumerPorts( ComponentType component ) {
        LinkedList< ConsumerPort > result = new LinkedList< ConsumerPort >();
        for( Port p : component.getPorts() ) {
            if( p instanceof ConsumerPort ) {
                   result.add(( ConsumerPort ) p );
            }
        }
        return result;
    }

    public List< ProducerPort > getComponentProducerPorts( ComponentType component ) {
        LinkedList< ProducerPort > result = new LinkedList< ProducerPort >();
        for( Port p : component.getPorts() ) {
            if( p instanceof ProducerPort ) {
                result.add(( ProducerPort ) p );
            }
        }
        return result;
    }

    public List< ConsumerPort > getComponentEventConsumerPorts( ComponentType component ) {
        LinkedList< ConsumerPort > result = new LinkedList< ConsumerPort >();
        for( Port p : component.getPorts() ) {
            if( p instanceof ConsumerPort ) {
                if( p.getTokenType() instanceof EventType ) {
                    result.add(( ConsumerPort ) p );
                }
            }
        }
        return result;
    }

    public List< ProducerPort > getComponentEventProducerPorts( ComponentType component ) {
        LinkedList< ProducerPort > result = new LinkedList< ProducerPort >();
        for( Port p : component.getPorts() ) {
            if( p instanceof ProducerPort ) {
                if( p.getTokenType() instanceof EventType ) {
                    result.add(( ProducerPort ) p );
                }
            }
        }
        return result;
    }

    public List< ConsumerPort > getComponentStreamConsumerPorts( ComponentType component ) {
        LinkedList< ConsumerPort > result = new LinkedList< ConsumerPort >();
        for( Port p : component.getPorts() ) {
            if( p instanceof ConsumerPort ) {
                if( p.getTokenType() instanceof StreamType ) {
                    result.add( ( ConsumerPort ) p );
                }
            }
        }
        return result;
    }

    public List< ProducerPort > getComponentStreamProducerPorts( ComponentType component ) {
        LinkedList< ProducerPort > result = new LinkedList< ProducerPort >();
        for( Port p : component.getPorts() ) {
            if( p instanceof ProducerPort ) {
                if( p.getTokenType() instanceof StreamType ) {
                    result.add(( ProducerPort ) p );
                }
            }
        }
        return result;
    }

    public HashSet< ConsumerPort > getConsumerPortsInExpression( Expression expr ) {
        HashSet< ConsumerPort > result = new HashSet< ConsumerPort >();
        getConsumerPortsInExpression( expr, result );
        return result;
    }

    private void getConsumerPortsInExpression( Expression expr, HashSet< ConsumerPort > result ) {
        if( expr instanceof SimpleExpression ) {
            getConsumerPortsInSimpleExpression( (SimpleExpression) expr, result );
        }
        else {
            getConsumerPortsInBinaryExpression( (BinaryExpression) expr, result );
        }
    }

    private void getConsumerPortsInSimpleExpression( SimpleExpression expr, HashSet< ConsumerPort > result ) {
        if( expr instanceof UnaryExpression ) {
            getConsumerPortsInExpression( ( (UnaryExpression) expr ).getExpression(), result );
        }

        else if( expr instanceof PortExpression ) {
        	Port p = (( PortExpression ) expr ).getPort();
        	if( p instanceof ConsumerPort ) {
        		result.add( ( ConsumerPort ) p );
        	}
        }
    }

    private void getConsumerPortsInBinaryExpression( BinaryExpression expr, HashSet< ConsumerPort > result ) {
        getConsumerPortsInExpression( expr.getLeftExpression(), result );
        getConsumerPortsInExpression( expr.getRightExpression(), result );
    }

    public static String getStringForExpression( Expression expr ) {
        if( expr instanceof SimpleExpression ) {
            return getStringForSimpleExpression( (SimpleExpression) expr );
        }
        else {
            return getStringForBinaryExpression( ( BinaryExpression ) expr );
        }
    }

    public static String getStringForUnaryOperator(UnaryOperator op) {
        switch(op.getValue()) {
        	case UnaryOperator.NONE_VALUE: 		 return "";
            case UnaryOperator.NOT_VALUE:        return "!";
            case UnaryOperator.ALWAYS_VALUE:     return "[]";
            case UnaryOperator.ONCE_VALUE:       return "<>";
            case UnaryOperator.PREVIOUS_VALUE:	 return "Prev";
            default:                             return op.getLiteral();
        }
    }

    private static String getStringForSimpleExpression( SimpleExpression expr ) {
        if( expr instanceof UnaryExpression ) {
            UnaryExpression uExpr = ( UnaryExpression )expr;
            return getStringForUnaryOperator( uExpr.getOperator() ) + " (" +
                    getStringForExpression( uExpr.getExpression() ) + ")";
        }
        else if( expr instanceof PortExpression ) {
        	PortExpression pe = (PortExpression) expr;
        	//String compName;
        	//Component comp = pe.getComponent();
        	//if(comp == null) compName = "<app>"; else compName = comp.getName();
            //return compName + "." + pe.getPort().getName();
            return pe.getPort().getName();
        }
        else if( expr instanceof LiteralExpression ) {
            return (( LiteralExpression ) expr ).getValue();
        }
        else if( expr instanceof ActiveExpression ) {
            ActiveExpression ap = (ActiveExpression)expr;
            return
                (ap.isActive() ? "" : "in") + "active(" +
                ap.getComponent().getName() + ")";
        }
        else {
            return "";
        }
    }

/*    public static String getStringForPort(Port port) {
        ComponentType component = port.getComponent();
        return
            (component == null ? "" : component.getName() + ".") + port.getName();
    }*/

    private static String getStringForBinaryExpression( BinaryExpression expr ) {
        return getStringForExpression( expr.getLeftExpression() )
             + " " + getStringForBinaryOperator( expr.getOperator() ) + " "
             + getStringForExpression( expr.getRightExpression() );
    }

    private static String getStringForBinaryOperator( BinaryOperator op ) {
        switch( op.getValue() ) {
            case BinaryOperator.EQUAL_VALUE:         		return "==";
            case BinaryOperator.LESS_EQUAL_VALUE:    		return "<=";
            case BinaryOperator.LESS_VALUE:          		return "<";
            case BinaryOperator.GREATER_VALUE:       		return ">";
            case BinaryOperator.GREATER_EQUAL_VALUE: 		return ">=";
            case BinaryOperator.NOT_EQUAL_VALUE:     		return "!=";
            case BinaryOperator.CONNECTED_TO_FROM_VALUE:	return "<<";
            case BinaryOperator.OR_VALUE:          		return "||";
            case BinaryOperator.AND_VALUE:         		return "&&";
            case BinaryOperator.BACK_TO_VALUE:      		return "backto";
            case BinaryOperator.IMPLIES_VALUE:				return "=>";
            case BinaryOperator.SINCE_VALUE:       		return "since";
           default:                            				return op.getLiteral();
        }
    }

    public Application getApplication() {
        return application_;
    }

    void setApplication( Application application ) {
        application_ = application;
    }

    /**
     * The application
     */
    private Application application_ = null;

    /**
     * The modules
     */
    private HashMap< String, Module > modules_ = new HashMap< String, Module >();

}
