/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.parser;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import antlr.ANTLRException;
import antlr.CommonAST;
import antlr.debug.misc.ASTFrame;

class TestADLVParser {
	
	public static void main( String[] args ) {
		System.out.println( "Debut..." );
		try {
			// Création du couple parser/lexer
			ADLVLexer lexer = new ADLVLexer( new BufferedInputStream( new
					FileInputStream( "../SpeedRegul/SpeedControl.adlv" )));
			ADLVParser parser = new ADLVParser( lexer );
			parser.setASTNodeClass(NumberedAST.class.getName());
			// lancement de l'analyse depuis la racine
			try {
				parser.specification();
			} catch( ANTLRException ex ) {
				System.err.println( "Error during parsing : " + ex.getMessage() );
			}
			System.out.println( "Fin !" );
			// affichage des erreurs le cas échéant
			showErrors( lexer, parser );
			// affichage de l'AST (indicatif, pas vraiment utilisé)
			showAST( parser, true, true );
		} catch( FileNotFoundException ex ) {
			System.err.println( "Source file not found : " + ex.toString() );
		}
	}	
	private static void showAST( 
			ADLVParser parser,
			boolean showText,
			boolean showASTFrame ) {
		CommonAST ast = ( CommonAST ) parser.getAST();
		if(ast == null) {
			System.err.println( "Cannot create AST" );
		}
		else {
			if( showText )
				System.out.println( ast.toStringList() );
			if( showASTFrame ) {
				antlr.debug.misc.ASTFrame frame = new ASTFrame( "AST", ast );
				frame.setVisible( true );
			}
		}
	}

	private static void showErrors( ADLVLexer lexer, ADLVParser parser ) {
		if (lexer.errorNumber() != 0
				||      lexer.warningNumber() != 0
				||      parser.errorNumber() != 0
				||      parser.warningNumber() != 0) {
			String [] lwarnings = lexer.warnings();
			String [] lerrors = lexer.errors();
			String [] pwarnings = parser.warnings();
			String [] perrors = parser.errors();
			StringBuffer ret = new StringBuffer();
			for( int i = 0; i < lwarnings.length; ++i ) {
				ret.append( "Warning: " );
				ret.append( lwarnings[i] );
				ret.append( '\n' );
			}
			for( int i = 0; i < pwarnings.length; ++i ) {
				ret.append( "Warning: " );
				ret.append( pwarnings[i] );
				ret.append( '\n' );
			}
			for( int i = 0; i < lerrors.length; ++i ) {
				ret.append( "Error :" );
				ret.append( lerrors[i] );
				ret.append( '\n' );
			}
			for( int i = 0; i < perrors.length; ++i ) {
				ret.append( "Error :" );
				ret.append( perrors[i] );
				ret.append( '\n' );
			}
			ret.append( "Parsing: " + ( lwarnings.length + pwarnings.length ) +
					" warnings and "	+ ( lerrors.length + perrors.length ) +
					" errors." );
			System.err.println( ret.toString() );
		}
	}
}

